@@ -23,25 +23,37 @@
 #define __AA_PARSER_H
 
 
+#include <endian.h>
 #include <string.h>
-#include <netinet/in.h>
 #include <sys/resource.h>
+
+#include <libintl.h>
+#define _(s) gettext(s)
+
 #include "immunix.h"
 #include "libapparmor_re/apparmor_re.h"
 #include "libapparmor_re/aare_rules.h"
 
+#include <string>
+
 using namespace std;
 
 #include <set>
 class Profile;
-
-struct mnt_ent;
+class rule_t;
 
 /* Global variable to pass token to lexer.  Will be replaced by parameter
  * when lexer and parser are made reentrant
  */
 extern int parser_token;
 
+
+#define WARN_RULE_NOT_ENFORCED	1
+#define WARN_RULE_DOWNGRADED	2
+
+extern dfaflags_t warnflags;
+
+
 typedef enum pattern_t pattern_t;
 
 struct prefixes {
@@ -75,6 +87,12 @@ struct cond_entry {
 	struct cond_entry *next;
 };
 
+struct cond_entry_list {
+	char *name;
+
+	struct cond_entry *list;
+};
+
 struct cod_entry {
 	char *ns;
 	char *name;
@@ -96,15 +114,6 @@ struct cod_entry {
 	struct cod_entry *next;
 };
 
-/* supported AF protocols */
-struct aa_network_entry {
-	unsigned int family;
-	unsigned int type;
-	unsigned int protocol;
-
-	struct aa_network_entry *next;
-};
-
 struct aa_rlimits {
 	unsigned int specified;			/* limits that are set */
 	rlim_t limits[RLIMIT_NLIMITS];
@@ -120,18 +129,6 @@ struct sd_hat {
 	unsigned int hat_magic;
 };
 
-/* describe an ip address */
-struct ipv4_desc {
-	struct in_addr addr;
-	unsigned long mask;
-	unsigned short port[2];
-};
-
-struct ipv4_endpoints {
-	struct ipv4_desc * src;
-	struct ipv4_desc * dest;
-};
-
 struct var_string {
 	char *prefix;
 	char *var;
@@ -193,8 +190,8 @@ extern int preprocess_only;
 #define MIN_PORT 0
 #define MAX_PORT 65535
 
-#ifndef __unused
-#define __unused __attribute__ ((unused))
+#ifndef unused
+#define unused __attribute__ ((unused))
 #endif
 
 
@@ -211,6 +208,40 @@ extern int preprocess_only;
 		___tmp->next = (LISTB);		\
 	} while (0)
 
+#define list_len(LIST)		\
+({				\
+	int len = 0;		\
+	typeof(LIST) tmp;		\
+	list_for_each((LIST), tmp)	\
+		len++;		\
+	len;			\
+})
+
+#define list_find_prev(LIST, ENTRY)	\
+({					\
+	typeof(ENTRY) tmp, prev = NULL;	\
+	list_for_each((LIST), tmp) {	\
+		if (tmp == (ENTRY))	\
+			break;		\
+		prev = tmp;		\
+	}				\
+	prev;				\
+})
+
+#define list_remove_at(LIST, PREV, ENTRY)			\
+	if (PREV)						\
+		(PREV)->next = (ENTRY)->next;			\
+	if ((ENTRY) == (LIST))					\
+		(LIST) = (ENTRY)->next;				\
+	(ENTRY)->next = NULL;					\
+
+#define list_remove(LIST, ENTRY)				\
+do {								\
+	typeof(ENTRY) prev = list_find_prev((LIST), (ENTRY));	\
+	list_remove_at((LIST), prev, (ENTRY));			\
+} while (0)
+
+
 #define DUP_STRING(orig, new, field, fail_target) \
 	do {									\
 		(new)->field = ((orig)->field) ? strdup((orig)->field) : NULL;	\
@@ -218,13 +249,56 @@ extern int preprocess_only;
 				goto fail_target;				\
 	} while (0)
 
+
+#define u8  unsigned char
+#define u16 uint16_t
+#define u32 uint32_t
+#define u64 uint64_t
+
+#define cpu_to_le16(x) ((u16)(htole16 ((u16) x)))
+#define cpu_to_le32(x) ((u32)(htole32 ((u32) x)))
+#define cpu_to_le64(x) ((u64)(htole64 ((u64) x)))
+
+/* The encoding for kernal abi > 5 is
+ * 28-31: reserved
+ * 20-27: policy version
+ * 12-19: policy abi version
+ * 11:    force complain flag
+ * 10:    reserved
+ * 0-9:   kernel abi version
+ */
+#define ENCODE_VERSION(C, P, PABI, KABI)		\
+({							\
+	u32 version = (KABI) & 0x3ff;			\
+	if ((KABI) > 5) {				\
+		version |= (C) ? 1 << 11 : 0;		\
+		version |= ((PABI) & 0xff) << 12;	\
+		version |= ((P) & 0xff) << 20;		\
+	}						\
+	version;					\
+})
+
+/* The parser fills this variable in automatically */
+#define PROFILE_NAME_VARIABLE "profile_name"
+
 /* from parser_common.c */
+extern uint32_t policy_version;
+extern uint32_t parser_abi_version;
+extern uint32_t kernel_abi_version;
+
+extern int force_complain;
 extern int perms_create;
 extern int net_af_max_override;
 extern int kernel_load;
+extern int kernel_supports_setload;
 extern int kernel_supports_network;
+extern int kernel_supports_policydb;
+extern int kernel_supports_diff_encode;
 extern int kernel_supports_mount;
 extern int kernel_supports_dbus;
+extern int kernel_supports_signal;
+extern int kernel_supports_ptrace;
+extern int kernel_supports_unix;
 extern int conf_verbose;
 extern int conf_quiet;
 extern int names_only;
@@ -244,6 +318,7 @@ extern void pwarn(const char *fmt, ...) __attribute__((__format__(__printf__, 1,
 extern int force_complain;
 extern struct timespec mru_tstamp;
 extern void update_mru_tstamp(FILE *file);
+extern void display_version(void);
 
 /* provided by parser_lex.l (cannot be used in tst builds) */
 extern FILE *yyin;
@@ -256,9 +331,16 @@ extern int yylex(void);
 extern const char *basedir;
 
 /* parser_regex.c */
+#define default_match_pattern "[^\\000]*"
+#define anyone_match_pattern "[^\\000]+"
+
+extern pattern_t convert_aaregex_to_pcre(const char *aare, int anchor,
+					 std::string& pcre, int *first_re_pos);
+extern int build_list_val_expr(std::string& buffer, struct value_list *list);
+extern int convert_entry(std::string& buffer, char *entry);
+extern int clear_and_convert_entry(std::string& buffer, char *entry);
 extern int process_regex(Profile *prof);
 extern int post_process_entry(struct cod_entry *entry);
-extern int process_dbus(Profile *prof);
 
 extern void reset_regex(void);
 
@@ -267,17 +349,21 @@ extern int process_policydb(Profile *prof);
 extern int process_policy_ents(Profile *prof);
 
 /* parser_variable.c */
+int expand_entry_variables(char **name);
 extern int process_variables(Profile *prof);
 extern struct var_string *split_out_var(const char *string);
 extern void free_var_string(struct var_string *var);
 
 /* parser_misc.c */
+extern void warn_uppercase(void);
 extern int is_blacklisted(const char *name, const char *path);
 extern struct value_list *new_value_list(char *value);
 extern struct value_list *dup_value_list(struct value_list *list);
 extern void free_value_list(struct value_list *list);
 extern void print_value_list(struct value_list *list);
 extern struct cond_entry *new_cond_entry(char *name, int eq, struct value_list *list);
+extern void move_conditional_value(const char *rulename, char **dst_ptr,
+				   struct cond_entry *cond_ent);
 extern void free_cond_entry(struct cond_entry *ent);
 extern void free_cond_list(struct cond_entry *ents);
 extern void print_cond_entry(struct cond_entry *ent);
@@ -289,29 +375,19 @@ extern int name_to_capability(const char *keyword);
 extern int get_rlimit(const char *name);
 extern char *process_var(const char *var);
 extern int parse_mode(const char *mode);
-extern int parse_dbus_mode(const char *str_mode, int *mode, int fail);
+extern int parse_X_mode(const char *X, int valid, const char *str_mode, int *mode, int fail);
 extern struct cod_entry *new_entry(char *ns, char *id, int mode, char *link_id);
-extern struct aa_network_entry *new_network_ent(unsigned int family,
-						unsigned int type,
-						unsigned int protocol);
-extern struct aa_network_entry *network_entry(const char *family,
-					      const char *type,
-					      const char *protocol);
-extern size_t get_af_max(void);
 
 /* returns -1 if value != true or false, otherwise 0 == false, 1 == true */
 extern int str_to_boolean(const char* str);
 extern struct cod_entry *copy_cod_entry(struct cod_entry *cod);
 extern void free_cod_entries(struct cod_entry *list);
-extern void free_mnt_entries(struct mnt_entry *list);
-extern void free_dbus_entries(struct dbus_entry *list);
 extern void __debug_capabilities(uint64_t capset, const char *name);
-void __debug_network(unsigned int *array, const char *name);
 void debug_cod_entries(struct cod_entry *list);
 
 
 /* parser_symtab.c */
-struct set_value {;
+struct set_value {
 	char *val;
 	struct set_value *next;
 };
@@ -321,6 +397,7 @@ extern int new_set_var(const char *var, const char *value);
 extern int add_set_value(const char *var, const char *value);
 extern struct set_value *get_set_var(const char *var);
 extern char *get_next_set_value(struct set_value **context);
+extern int delete_set_var(const char *var_name);
 extern void dump_symtab(void);
 extern void dump_expanded_symtab(void);
 void free_symtabs(void);
@@ -334,9 +411,8 @@ extern void free_aliases(void);
 extern int profile_merge_rules(Profile *prof);
 
 /* parser_interface.c */
-typedef struct __sdserialize sd_serialize;
 extern int load_profile(int option, Profile *prof);
-extern int sd_serialize_profile(sd_serialize *p, Profile *prof,
+extern void sd_serialize_profile(std::ostringstream &buf, Profile *prof,
 				int flatten);
 extern int sd_load_buffer(int option, char *buffer, int size);
 extern int cache_fd;
@@ -345,9 +421,10 @@ extern int cache_fd;
 /* parser_policy.c */
 extern void add_to_list(Profile *profile);
 extern void add_hat_to_policy(Profile *policy, Profile *hat);
+extern int add_entry_to_x_table(Profile *prof, char *name);
 extern void add_entry_to_policy(Profile *policy, struct cod_entry *entry);
 extern void post_process_file_entries(Profile *prof);
-extern void post_process_mnt_entries(Profile *prof);
+extern void post_process_rule_entries(Profile *prof);
 extern int post_process_policy(int debug_only);
 extern int process_profile_regex(Profile *prof);
 extern int process_profile_variables(Profile *prof);
@@ -356,7 +433,7 @@ extern int post_merge_rules(void);
 extern int merge_hat_rules(Profile *prof);
 extern Profile *merge_policy(Profile *a, Profile *b);
 extern int load_policy(int option);
-extern int load_hats(sd_serialize *p, Profile *prof);
+extern int load_hats(std::ostringstream &buf, Profile *prof);
 extern int load_flattened_hats(Profile *prof, int option);
 extern void dump_policy_hats(Profile *prof);
 extern void dump_policy_names(void);
