@@ -44,11 +44,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
-#include <libintl.h>
 #include <dirent.h>
 #include "parser.h"
 #include "parser_include.h"
-#define _(s) gettext(s)
 
 /* An array of search directories, I sure hope 100's enough */
 #define MAX_PATH 100
@@ -73,9 +71,9 @@ static char *stripblanks(char *s);
 /* default base directory is /etc/subdomain.d, it can be overriden
    with the -b option. */
 
-char *basedir;
-static char *default_basedir = "/etc/apparmor.d";
-static char *old_basedir = "/etc/subdomain.d";
+const char *basedir;
+static const char *default_basedir = "/etc/apparmor.d";
+static const char *old_basedir = "/etc/subdomain.d";
 
 
 /* set up basedir so that it can be overridden/used later. */
@@ -130,16 +128,22 @@ void set_base_dir(char *dir)
 }
 
 /* Add a directory to the search path. */
-int add_search_dir(char *dir)
+int add_search_dir(const char *dir)
 {
 	char *t;
+	size_t len;
+
 	if (npath >= MAX_PATH) {
 		PERROR(_("Error: Could not add directory %s to search path.\n"),
 		       dir);
 		return 0;
 	}
 
-	if (!dir || strlen(dir) <= 0)
+	if (!dir)
+		return 1;
+
+	len = strlen(dir);
+	if (len == 0)
 		return 1;
 
 	t = strdup(dir);
@@ -149,8 +153,8 @@ int add_search_dir(char *dir)
 	}
 
 	/*strip trailing /'s */
-	while (t[strlen(t) - 1] == '/')
-		t[strlen(t) - 1] = 0;
+	while (len > 0 && t[--len] == '/')
+		t[len] = '\0';
 	path[npath] = t;
 	npath++;
 
@@ -279,7 +283,7 @@ struct include_stack_t {
 
 struct include_stack_t *include_stack_head = NULL;
 
-static void start_include_position(char *filename)
+static void start_include_position(const char *filename)
 {
 	if (current_filename)
 		free(current_filename);
@@ -291,7 +295,7 @@ void push_include_stack(char *filename)
 {
 	struct include_stack_t *include = NULL;
 
-	include = malloc(sizeof(*include));
+	include = (struct include_stack_t *) malloc(sizeof(*include));
 	if (!include) {
 		perror("malloc of included file stack tracker");
 		/* failures in this area are non-fatal */
@@ -323,7 +327,7 @@ void pop_include_stack(void)
 	free(include);
 }
 
-void reset_include_stack(char *filename)
+void reset_include_stack(const char *filename)
 {
 	while (include_stack_head)
 		pop_include_stack();
